/* 
  TouchShield.h
  
  This file is part of TouchShield library for SparkFun Touch Shield.
  
  TouchShield library was developed by Synapse(Hiroshi Tanigawa) in 2016.
  This Library is  originally distributed at "Synapse's history of making
  gadgets." <https://synapse.kyoto>

  TouchShield library is free software: you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as published by
  the Free Software Foundation, either version 2.1 of the License, or
  (at your option) any later version.

  TouchShield library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public License
  along with TouchShield library.  If not, see <http://www.gnu.org/licenses/>.

  Some codes in this file were taken from the Touch Shield Example sketch
  that is distributed at Github (https://github.com/sparkfun/Touch_Shield).
  Touch Shield Example sketch was created by SparkFun Electronics.
*/

// include the atmel I2C libs
#include <arduino.h>
#include <Wire.h>
#include "utility/mpr121.h"
#include "utility/i2c.h"
#include "TouchShield.h"

using namespace TouchShieldSynapse;

namespace {

const int buf_size=16;
volatile char buf[buf_size];
volatile uint16_t buf_pos;
volatile uint16_t buf_len;
const uint8_t PadMap[9]={8,5,2,7,4,1,6,3,0};
char KeyMap[9];

void TouchShieldISR()
{
  uint16_t touchstatus;
  char c;

  EIMSK&=1; // disable INT0
  sei(); // enable interrupts

  touchstatus = mpr121Read(0x01) << 8;
  touchstatus |= mpr121Read(0x00);

  cli(); // disable interrupts
  EIMSK|=1; // enable INT0

  int cnt = 0;
  for (int i=0; i<9; i++) {
    if ((touchstatus & (1<<i))) {
      c=KeyMap[PadMap[i]];
      cnt++;
    } // if
  } // for i

  if (cnt==1 && buf_len<buf_size) {
    buf[(buf_pos+buf_len)%buf_size]=c;
    buf_len++;
  } // if
} // TouchShieldISR

} // namespace

TouchShield::TouchShield(char key1, char key2, char key3, char key4, char key5, char key6, char key7, char key8, char key9)
{
  KeyMap[0]=key1;
  KeyMap[1]=key2;
  KeyMap[2]=key3;
  KeyMap[3]=key4;
  KeyMap[4]=key5;
  KeyMap[5]=key6;
  KeyMap[6]=key7;
  KeyMap[7]=key8;
  KeyMap[8]=key9;
} // TouchShield::TouchShield

void TouchShield::init()
{
  const int IrqPin=2;

  pinMode(IrqPin, INPUT);
  digitalWrite(IrqPin, HIGH);

  Wire.begin();

  delay(100);

  // initialize mpr121
  mpr121QuickConfig();
  
  // initilaize buffer
  buf_pos=buf_len=0;

  // Create and interrupt to trigger when a button
  // is hit, the IRQ pin goes low, and the function TouchShieldISR is run. 
  attachInterrupt(0,TouchShieldISR,FALLING);
} // TouchShield::init

char TouchShield::read()
{
  if(buf_len==0) {
    return '\0';
  } // if
  
  char c=buf[buf_pos];
  buf_pos=(buf_pos+1)%buf_size;
  buf_len--;
  return c;
} // TouchShield::read
